@@ -99,7 +99,7 @@ typedef char s_char;
 /*
  * Poll interval parameters
  */
-#define NTP_UNREACH	16	/* poll interval backoff count */
+#define NTP_UNREACH	12	/* poll unreach threshold */
 #define	NTP_MINPOLL	4	/* log2 min poll interval (16 s) */
 #define NTP_MINDPOLL	6	/* log2 default min poll (64 s) */
 #define NTP_MAXDPOLL	10	/* log2 default max poll (~17 m) */
@@ -119,28 +119,27 @@ typedef char s_char;
 /*
  * Selection algorithm tuning parameters
  */
-#define	NTP_MINCLOCK	3	/* minimum survivors */
-#define	NTP_MAXCLOCK	10	/* maximum candidates */
+#define	NTP_MINCLOCK	3	/* min cluster survivors */
+#define	NTP_MAXCLOCK	50	/* max selection candidates */
 #define MAXDISTANCE	1.	/* max root distance */
 #define CLOCK_SGATE	3.	/* popcorn spike gate */
 #define HUFFPUFF	900	/* huff-n'-puff sample interval (s) */
 #define HYST		.5	/* anti-clockhop hysteresis */
 #define HYST_TC		.875	/* anti-clockhop hysteresis decay */
 #define MAX_TTL		8	/* max ttl mapping vector size */
-#define NTP_MAXEXTEN	1024	/* maximum extension field size */
+#define NTP_MAXEXTEN	1024	/* max extension field size */
 
 /*
  * Miscellaneous stuff
  */
-#define NTP_MAXKEY	65535	/* maximum authentication key number */
+#define NTP_MAXKEY	65535	/* max authentication key number */
 
 /*
  * Limits of things
  */
 #define	MAXFILENAME	128	/* max length of file name */
-#define MAXHOSTNAME	128	/* max length of DNS host name */
-#define MINHOSTNAME	4	/* min length of DNS host name */
-#define NTP_MAXSTRLEN	256	/* maximum string length */
+#define MAXHOSTNAME	512	/* max length of host/node name */
+#define NTP_MAXSTRLEN	256	/* max string length */
 #define MAXINTERFACES	512	/* max number of interfaces */
 
 /*
@@ -174,22 +173,27 @@ struct interface {
 	struct sockaddr_storage sin;	/* interface address */
 	struct sockaddr_storage bcast; /* broadcast address */
 	struct sockaddr_storage mask; /* interface mask */
-	char name[8];		/* name of interface */
+	char name[32];		/* name of interface */
 	int flags;		/* interface flags */
 	int last_ttl;		/* last TTL specified */
 	u_int addr_refid;	/* IPv4 addr or IPv6 hash */
+	int num_mcast;		/* No. of IP addresses in multicast socket */
 	volatile long received;	/* number of incoming packets */
 	long sent;		/* number of outgoing packets */
 	long notsent;		/* number of send failures */
+	u_int ifindex;		/* Interface index */
+	u_int scopeid;		/* Scope used for Multicasting */
 };
 
 /*
  * Flags for interfaces
  */
-#define	INT_BROADCAST	1	/* can broadcast out this interface */
-#define	INT_BCASTOPEN	2	/* broadcast socket is open */
-#define	INT_LOOPBACK	4	/* the loopback interface */
-#define INT_MULTICAST	8	/* multicasting enabled */
+#define INT_UP		 1	/* Interface is up */
+#define	INT_PPP		 2	/* Point-to-point interface */
+#define	INT_LOOPBACK	 4	/* the loopback interface */
+#define	INT_BROADCAST	 8	/* can broadcast out this interface */
+#define INT_MULTICAST	16	/* multicasting enabled */
+#define	INT_BCASTOPEN	32	/* broadcast socket is open */
 
 /*
  * Define flasher bits (tests 1 through 11 in packet procedure)
@@ -201,13 +205,13 @@ struct interface {
 #define TEST2		0x0002	/* bogus packet received */
 #define TEST3		0x0004	/* protocol unsynchronized */
 #define TEST4		0x0008	/* access denied */
-#define TEST5		0x0010	/* authentication failed */
+#define TEST5		0x0010	/* MAC error */
 #define TEST6		0x0020	/* peer clock unsynchronized */
 #define TEST7		0x0040	/* peer stratum out of bounds */
 #define TEST8		0x0080  /* root delay/dispersion bounds check */
 #define TEST9		0x0100	/* peer delay/dispersion bounds check */
-#define TEST10		0x0200	/* autokey failed */
-#define	TEST11		0x0400	/* proventic not confirmed */
+#define TEST10		0x0200	/* autokey error */
+#define	TEST11		0x0400	/* protocol error */
 
 /*
  * The peer structure. Holds state information relating to the guys
@@ -223,10 +227,8 @@ struct peer {
 	u_char	version;	/* version number */
 	u_char	hmode;		/* local association mode */
 	u_char	hpoll;		/* local poll interval */
-	u_char	kpoll;		/* last poll interval */
 	u_char	minpoll;	/* min poll interval */
 	u_char	maxpoll;	/* max poll interval */
-	u_char	burst;		/* packets remaining in burst */
 	u_int	flags;		/* association flags */
 	u_char	cast_flags;	/* additional flags */
 	u_int	flash;		/* protocol error test tally bits */
@@ -237,10 +239,12 @@ struct peer {
 	/*
 	 * Variables used by reference clock support
 	 */
+#ifdef REFCLOCK
 	struct refclockproc *procptr; /* refclock structure pointer */
 	u_char	refclktype;	/* reference clock type */
 	u_char	refclkunit;	/* reference clock unit number */
 	u_char	sstclktype;	/* clock type for system status word */
+#endif /* REFCLOCK */
 
 	/*
 	 * Variables set by received packet
@@ -263,8 +267,8 @@ struct peer {
 	u_int32	crypto;		/* peer status word */
 	EVP_PKEY *pkey;		/* public key */
 	const EVP_MD *digest;	/* message digest algorithm */
-	u_char	*subject;	/* certificate subject name */
-	u_char	*issuer;	/* certificate issuer name */
+	char	*subject;	/* certificate subject name */
+	char	*issuer;	/* certificate issuer name */
 	keyid_t	pkeyid;		/* previous key ID */
 	keyid_t	pcookie;	/* peer cookie */
 	EVP_PKEY *ident_pkey;	/* identity key */
@@ -293,6 +297,7 @@ struct peer {
 	u_char	status;		/* peer status */
 	u_char	reach;		/* reachability register */
 	u_long	epoch;		/* reference epoch */
+	u_int	burst;		/* packets remaining in burst */
 	u_short	filter_nextpt;	/* index into filter shift register */
 	double	filter_delay[NTP_SHIFT]; /* delay shift register */
 	double	filter_offset[NTP_SHIFT]; /* offset shift register */
@@ -340,6 +345,7 @@ struct peer {
 	u_long	oldpkt;		/* packets duplicate packet */
 	u_long	seldisptoolarge; /* packets dispersion to large*/
 	u_long	selbroken;	/* not used */
+	u_long	rank;	/* number of times selected or in cluster */
 };
 
 /*
@@ -480,7 +486,7 @@ struct peer {
  	(sizeof(struct sockaddr_in)) : (sizeof(struct sockaddr_in6)))
 
 #define ANYSOCK(sock) \
-	memset(((struct sockaddr_in *)sock), 0, \
+	memset(((struct sockaddr_storage *)sock), 0, \
 	    sizeof(struct sockaddr_storage))
 
 #define ANY_INTERFACE_CHOOSE(sock) \
@@ -653,9 +659,9 @@ struct pkt {
  * To speed lookups, peers are hashed by the low order bits of the
  * remote IP address. These definitions relate to that.
  */
-#define	HASH_SIZE	128
-#define	HASH_MASK	(HASH_SIZE-1)
-#define	HASH_ADDR(src)	sock_hash(src)
+#define	NTP_HASH_SIZE	128
+#define	NTP_HASH_MASK	(NTP_HASH_SIZE-1)
+#define	NTP_HASH_ADDR(src)	sock_hash(src)
 
 /*
  * How we randomize polls.  The poll interval is a power of two.
@@ -722,6 +728,7 @@ struct pkt {
 #define LOOP_MINPOLL		7	/* set min poll interval (log2 s) */
 #define LOOP_ALLAN		8	/* set minimum Allan intercept */
 #define LOOP_HUFFPUFF		9	/* set huff-n'-puff filter length */
+#define LOOP_FREQ		10	/* set initial frequency */
 
 /*
  * Configuration items for the stats printer
@@ -747,8 +754,7 @@ struct mon_data {
 	struct mon_data *mru_prev;	/* previous structure in MRU list */
 	u_long drop_count;		/* dropped due RESLIMIT*/
 	double avg_interval;		/* average interpacket interval */
-	u_long lasttime;		/* last packet received time */
-	u_long firsttime;		/* first packet received time */
+	u_long lasttime;		/* interval since last packet */
 	u_long count;			/* total packet count */
 	struct sockaddr_storage rmtadr;	/* address of remote host */
 	struct interface *interface;	/* interface on which this arrived */
