@@ -81,7 +81,7 @@ extern	void	io_setbclient	P((void));
 extern	void	io_unsetbclient P((void));
 extern	void	io_multicast_add P((struct sockaddr_storage));
 extern	void	io_multicast_del P((struct sockaddr_storage));
-extern	void	kill_asyncio	 P((void));
+extern	void	kill_asyncio	 P((int));
 
 extern	void	sendpkt 	P((struct sockaddr_storage *, struct interface *, int, struct pkt *, int));
 #ifdef HAVE_SIGNALED_IO
@@ -104,7 +104,7 @@ extern	int	leap_actual P((int));
 
 /* ntp_loopfilter.c */
 extern	void	init_loopfilter P((void));
-extern	int 	local_clock P((struct peer *, double, double));
+extern	int 	local_clock P((struct peer *, double));
 extern	void	adj_host_clock	P((void));
 extern	void	loop_config P((int, double));
 extern	void	huffpuff	P((void));
@@ -145,7 +145,7 @@ extern	void	crypto_update	P((void));
 extern	void	crypto_config	P((int, char *));
 extern	void	crypto_setup	P((void));
 extern	u_int	crypto_ident	P((struct peer *));
-extern	struct exten *crypto_args P((struct peer *, u_int, u_char *));
+extern	struct exten *crypto_args P((struct peer *, u_int, char *));
 extern	int	crypto_public	P((struct peer *, u_char *, u_int));
 extern	void	value_free	P((struct value *));
 #endif /* OPENSSL */
@@ -192,6 +192,7 @@ extern	void	hack_restrict	P((int, struct sockaddr_storage *, struct sockaddr_sto
 
 /* ntp_timer.c */
 extern	void	init_timer	P((void));
+extern	void	reinit_timer	P((void));
 extern	void	timer		P((void));
 extern	void	timer_clr_stats P((void));
 #ifdef OPENSSL
@@ -314,8 +315,8 @@ extern int	tc_counter;		/* poll-adjust counter */
 extern u_long	last_time;		/* time of last clock update (s) */
 extern double	last_offset;		/* last clock offset (s) */
 extern double	allan_xpt;		/* Allan intercept (s) */
-extern double	sys_error;		/* system RMS error (s) */
-extern double	sys_jitter;		/* system RMS jitter (s) */
+extern double	sys_error;		/* system error (s) */
+extern double	sys_jitter;		/* system jitter (s) */
 
 /* ntp_monitor.c */
 extern struct mon_data mon_mru_list;
@@ -401,6 +402,7 @@ extern struct restrictlist *restrictlist; /* the ipv4 restriction list */
 extern struct restrictlist6 *restrictlist6; /* the ipv6 restriction list */
 extern u_long	res_min_interval;
 extern u_long	res_avg_interval;
+extern u_long	mon_age;		/* monitor preempt age */
 
 /* ntp_timer.c */
 extern volatile int alarm_flag;		/* alarm flag */
@@ -418,7 +420,8 @@ extern int	stats_control;		/* write stats to fileset? */
 extern volatile int debug;		/* debugging flag */
 extern int	nofork;			/* no-fork flag */
 extern int 	initializing;		/* initializing flag */
-#ifdef HAVE_CLOCKCTL
+#ifdef HAVE_DROPROOT
+extern int droproot;			/* flag: try to drop root privileges after startup */
 extern char *user;			/* user to switch to */
 extern char *group;			/* group to switch to */
 extern char *chrootdir;			/* directory to chroot to */
