@@ -22,13 +22,14 @@ humandate(
 {
 	char *bp;
 	struct tm *tm;
-	time_t sec;
+
+	tm = ntp2unix_tm(ntptime, 1);
+
+	if (!tm)
+		return "--- --- -- ---- --:--:--";
 
 	LIB_GETBUF(bp);
 	
-	sec = ntptime - JAN_1970;
-	tm = localtime(&sec);
-
 	(void) sprintf(bp, "%s, %s %2d %4d %2d:%02d:%02d",
 		       days[tm->tm_wday], months[tm->tm_mon], tm->tm_mday,
 		       1900+tm->tm_year, tm->tm_hour, tm->tm_min, tm->tm_sec);
@@ -45,8 +46,12 @@ humanlogtime(void)
 {
 	char *bp;
 	time_t cursec = time((time_t *) 0);
-	struct tm *tm = localtime(&cursec);
+	struct tm *tm;
 	
+	tm = localtime(&cursec);
+	if (!tm)
+		return "-- --- --:--:--";
+
 	LIB_GETBUF(bp);
 	
 	(void) sprintf(bp, "%2d %s %02d:%02d:%02d",
