@@ -213,6 +213,7 @@ static struct keyword flags_keywords[] = {
 static struct keyword discard_keywords[] = {
 	{ "average",		CONF_DISCARD_AVERAGE },
 	{ "minimum",		CONF_DISCARD_MINIMUM },
+	{ "monitor",		CONF_DISCARD_MONITOR },
 	{ "",			CONFIG_UNKNOWN }
 };
 
@@ -226,6 +227,7 @@ static struct keyword tinker_keywords[] = {
 	{ "stepout",		CONF_CLOCK_MINSTEP },
 	{ "allan",		CONF_CLOCK_ALLAN },
 	{ "huffpuff",		CONF_CLOCK_HUFFPUFF },
+	{ "freq",		CONF_CLOCK_FREQ },
 	{ "",			CONFIG_UNKNOWN }
 };
 
@@ -489,7 +491,7 @@ getconfig(
 	int includelevel = 0;
 	char line[MAXLINE];
 	char *(tokens[MAXTOKENS]);
-	int ntokens;
+	int ntokens = 0;
 	int tok = CONFIG_UNKNOWN;
 	struct interface *localaddr;
 	struct refclockstat clock_stat;
@@ -923,7 +925,11 @@ getconfig(
 			break;
 
 		    case CONFIG_BROADCASTCLIENT:
-			proto_config(PROTO_BROADCLIENT, 1, 0., NULL);
+			if (ntokens == 1) {
+				proto_config(PROTO_BROADCLIENT, 1, 0., NULL);
+			} else {
+				proto_config(PROTO_BROADCLIENT, 2, 0., NULL);
+			}
 			break;
 
 		    case CONFIG_MULTICASTCLIENT:
@@ -1020,6 +1026,10 @@ getconfig(
 			    case CONF_CLOCK_HUFFPUFF:
 				loop_config(LOOP_HUFFPUFF, ftemp);
 				break;
+
+			    case CONF_CLOCK_FREQ:
+				loop_config(LOOP_FREQ, ftemp);
+				break;  
 			    }
 			}
 			break;
@@ -1069,17 +1079,6 @@ getconfig(
 			}
 			break;
 
-#ifdef OPENSSL
-		    case CONFIG_REVOKE:
-			if (ntokens >= 2)
-			    sys_revoke = max(atoi(tokens[1]), KEY_REVOKE);
-			break;
-
-		    case CONFIG_AUTOMAX:
-			if (ntokens >= 2)
-			    sys_automax = 1 << max(atoi(tokens[1]), 10);
-			break;
-
 		    case CONFIG_DISCARD:
 			for (i = 1; i < ntokens; i++) {
 			    int temp;
@@ -1094,12 +1093,17 @@ getconfig(
 			    }
 			    switch(temp) {
 			    case CONF_DISCARD_AVERAGE:
-				res_avg_interval = atoi(tokens[i++]);
+				res_avg_interval = atoi(tokens[i]);
 				break;
 
 			    case CONF_DISCARD_MINIMUM:
-				res_min_interval = atoi(tokens[i++]);
+				res_min_interval = atoi(tokens[i]);
 				break;
+
+			    case CONF_DISCARD_MONITOR:
+				mon_age = atoi(tokens[i]);
+				break;
+
 			    default:
 				msyslog(LOG_ERR,
 				    "discard: unknown keyword");
@@ -1108,6 +1112,17 @@ getconfig(
 			}
 			break;
 
+#ifdef OPENSSL
+		    case CONFIG_REVOKE:
+			if (ntokens >= 2)
+			    sys_revoke = (u_char) max(atoi(tokens[1]), KEY_REVOKE);
+			break;
+
+		    case CONFIG_AUTOMAX:
+			if (ntokens >= 2)
+			    sys_automax = 1 << max(atoi(tokens[1]), 10);
+			break;
+
 		    case CONFIG_CRYPTO:
 			if (ntokens == 1) {
 				crypto_config(CRYPTO_CONF_NONE, NULL);
@@ -2065,23 +2080,28 @@ getnetnum(
 
 	/* Get host address. Looking for UDP datagram connection */
  	memset(&hints, 0, sizeof (hints));
- 	hints.ai_socktype = SOCK_DGRAM;
-	hints.ai_family = addr->ss_family;
+ 	if (addr->ss_family == AF_INET || addr->ss_family == AF_INET6)
+	    hints.ai_family = addr->ss_family;
+	else
+	    hints.ai_family = AF_UNSPEC;
 
+	hints.ai_socktype = SOCK_DGRAM;
 #ifdef DEBUG
 		if (debug > 3)
 			printf("getaddrinfo %s\n", num);
 #endif
-	if (getaddrinfo(num, "123", &hints, &ptr)!=0) {
+	if (getaddrinfo(num, "ntp", &hints, &ptr)!=0) {
 		if (complain)
 			msyslog(LOG_ERR,
-				"getaddrinfo: \"%s\" invalid host address, line ignored",
+				"getaddrinfo: \"%s\" invalid host address, ignored",
 				num);
 #ifdef DEBUG
 		if (debug > 3)
 			printf(
-				"getaddrinfo: \"%s\" invalid host address, line ignored\n",
-				num);
+				"getaddrinfo: \"%s\" invalid host address%s.\n",
+				num, (complain)
+				? ", ignored"
+				: "");
 #endif
 		return 0;
 	}
@@ -2092,6 +2112,7 @@ getnetnum(
 		printf("getnetnum given %s, got %s \n",
 		   num, stoa(addr));
 #endif
+        freeaddrinfo(ptr);
 	return 1;
 }
 
@@ -2278,7 +2299,7 @@ do_resolve_internal(void)
 		 */
 
 		closelog();
-		kill_asyncio();
+		kill_asyncio(0);
 
 		(void) signal_no_reset(SIGCHLD, SIG_DFL);
 
