@@ -42,7 +42,9 @@ struct ctl_proc {
  * Request processing routines
  */
 static	void	ctl_error	P((int));
+#ifdef REFCLOCK
 static	u_short ctlclkstatus	P((struct refclockstat *));
+#endif
 static	void	ctl_flushpkt	P((int));
 static	void	ctl_putdata	P((const char *, unsigned int, int));
 static	void	ctl_putstr	P((const char *, const char *,
@@ -104,23 +106,24 @@ static struct ctl_var sys_var[] = {
 	{ CS_OFFSET,	RO, "offset" },		/* 11 */
 	{ CS_DRIFT,	RO, "frequency" },	/* 12 */
 	{ CS_JITTER,	RO, "jitter" },		/* 13 */
-	{ CS_CLOCK,	RO, "clock" },		/* 14 */
-	{ CS_PROCESSOR, RO, "processor" },	/* 15 */
-	{ CS_SYSTEM,	RO, "system" },		/* 16 */
-	{ CS_VERSION,	RO, "version" },	/* 17 */
-	{ CS_STABIL,	RO, "stability" },	/* 18 */
-	{ CS_VARLIST,	RO, "sys_var_list" },	/* 19 */
+	{ CS_ERROR,	RO, "error" },		/* 14 */
+	{ CS_CLOCK,	RO, "clock" },		/* 15 */
+	{ CS_PROCESSOR, RO, "processor" },	/* 16 */
+	{ CS_SYSTEM,	RO, "system" },		/* 17 */
+	{ CS_VERSION,	RO, "version" },	/* 18 */
+	{ CS_STABIL,	RO, "stability" },	/* 19 */
+	{ CS_VARLIST,	RO, "sys_var_list" },	/* 20 */
 #ifdef OPENSSL
-	{ CS_FLAGS,	RO, "flags" },		/* 20 */
-	{ CS_HOST,	RO, "hostname" },	/* 21 */
-	{ CS_PUBLIC,	RO, "hostkey" },	/* 22 */
-	{ CS_CERTIF,	RO, "cert" },		/* 23 */
-	{ CS_REVTIME,	RO, "refresh" },	/* 24 */
-	{ CS_LEAPTAB,	RO, "leapseconds" },	/* 25 */
-	{ CS_TAI,	RO, "tai" },		/* 26 */
-	{ CS_DIGEST,	RO, "signature" },	/* 27 */
+	{ CS_FLAGS,	RO, "flags" },		/* 21 */
+	{ CS_HOST,	RO, "hostname" },	/* 22 */
+	{ CS_PUBLIC,	RO, "hostkey" },	/* 23 */
+	{ CS_CERTIF,	RO, "cert" },		/* 24 */
+	{ CS_REVTIME,	RO, "refresh" },	/* 25 */
+	{ CS_LEAPTAB,	RO, "leapseconds" },	/* 26 */
+	{ CS_TAI,	RO, "tai" },		/* 27 */
+	{ CS_DIGEST,	RO, "signature" },	/* 28 */
 #endif /* OPENSSL */
-	{ 0,		EOV, "" }		/* 28 */
+	{ 0,		EOV, "" }		/* 29 */
 };
 
 static struct ctl_var *ext_sys_var = (struct ctl_var *)0;
@@ -146,6 +149,7 @@ static	u_char def_sys_var[] = {
 	CS_STATE,
 	CS_OFFSET,
 	CS_DRIFT,
+	CS_ERROR,
 	CS_JITTER,
 	CS_STABIL,
 #ifdef OPENSSL
@@ -202,17 +206,18 @@ static struct ctl_var peer_var[] = {
 	{ CP_FILTERROR,	RO, "filtdisp=" },	/* 34 */
 	{ CP_FLASH,	RO, "flash" },		/* 35 */
 	{ CP_TTL,	RO, "ttl" },		/* 36 */
-	{ CP_VARLIST,	RO, "peer_var_list" },	/* 37 */
+	{ CP_RANK,	RO, "rank" },		/* 37 */
+	{ CP_VARLIST,	RO, "peer_var_list" },	/* 38 */
 #ifdef OPENSSL
-	{ CP_FLAGS,	RO, "flags" },		/* 38 */
-	{ CP_HOST,	RO, "hostname" },	/* 39 */
-	{ CP_INITSEQ,	RO, "initsequence" },   /* 40 */
-	{ CP_INITKEY,	RO, "initkey" },	/* 41 */
-	{ CP_INITTSP,	RO, "timestamp" },	/* 42 */
-	{ CP_DIGEST,	RO, "signature" },	/* 43 */
-	{ CP_IDENT,	RO, "identity" },	/* 44 */
+	{ CP_FLAGS,	RO, "flags" },		/* 39 */
+	{ CP_HOST,	RO, "hostname" },	/* 40 */
+	{ CP_INITSEQ,	RO, "initsequence" },   /* 41 */
+	{ CP_INITKEY,	RO, "initkey" },	/* 42 */
+	{ CP_INITTSP,	RO, "timestamp" },	/* 43 */
+	{ CP_DIGEST,	RO, "signature" },	/* 44 */
+	{ CP_IDENT,	RO, "identity" },	/* 45 */
 #endif /* OPENSSL */
-	{ 0,		EOV, "" }		/* 38/43 */
+	{ 0,		EOV, "" }		/* 39/46 */
 };
 
 
@@ -728,6 +733,7 @@ ctlpeerstatus(
 /*
  * ctlclkstatus - return a status word for this clock
  */
+#ifdef REFCLOCK
 static u_short
 ctlclkstatus(
 	struct refclockstat *this_clock
@@ -736,6 +742,7 @@ ctlclkstatus(
 	return ((u_short)(((this_clock->currentstatus) << 8) |
 	    (this_clock->lastevent)));
 }
+#endif
 
 
 /*
@@ -1232,6 +1239,10 @@ ctl_putsys(
 		ctl_putdbl(sys_var[CS_JITTER].text, sys_jitter * 1e3);
 		break;
 
+	case CS_ERROR:
+		ctl_putdbl(sys_var[CS_ERROR].text, sys_error * 1e3);
+		break;
+
 	case CS_CLOCK:
 		get_systime(&tmp);
 		ctl_putts(sys_var[CS_CLOCK].text, &tmp);
@@ -1527,6 +1538,10 @@ ctl_putpeer(
 		ctl_putuint(peer_var[CP_VALID].text, peer->unreach);
 		break;
 
+	case CP_RANK:
+		ctl_putuint(peer_var[CP_RANK].text, peer->rank);
+		break;
+
 	case CP_TIMER:
 		ctl_putuint(peer_var[CP_TIMER].text,
 		    peer->nextdate - current_time);
@@ -1894,10 +1909,10 @@ ctl_getitem(
 					if (cp < reqend)
 						cp++;
 					*tp-- = '\0';
-					while (tp > buf) {
-						*tp-- = '\0';
+					while (tp >= buf) {
 						if (!isspace((int)(*tp)))
 							break;
+						*tp-- = '\0';
 					}
 					reqpt = cp;
 					*data = buf;
@@ -1971,7 +1986,7 @@ read_status(
 
 		n = 0;
 		rpkt.status = htons(ctlsysstatus());
-		for (i = 0; i < HASH_SIZE; i++) {
+		for (i = 0; i < NTP_HASH_SIZE; i++) {
 			for (peer = assoc_hash[i]; peer != 0;
 				peer = peer->ass_next) {
 				ass_stat[n++] = htons(peer->associd);
@@ -2266,7 +2281,7 @@ read_clock_status(
 			peer = sys_peer;
 		} else {
 			peer = 0;
-			for (i = 0; peer == 0 && i < HASH_SIZE; i++) {
+			for (i = 0; peer == 0 && i < NTP_HASH_SIZE; i++) {
 				for (peer = assoc_hash[i]; peer != 0;
 					peer = peer->ass_next) {
 					if (peer->flags & FLAG_REFCLOCK)
