@@ -1,6 +1,8 @@
 /*
  * ntp_loopfilter.c - implements the NTP loop filter algorithm
  *
+ * ATTENTION: Get approval from Dave Mills on all changes to this file!
+ *
  */
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -148,7 +150,7 @@ int	state;			/* clock discipline state */
 int	tc_counter;		/* hysteresis counter */
 u_long	last_time;		/* time of last clock update (s) */
 double	last_offset;		/* last clock offset (s) */
-double	sys_jitter;		/* system RMS jitter (s) */
+double	sys_jitter;		/* system jitter (s) */
 
 /*
  * Huff-n'-puff filter variables
@@ -181,6 +183,7 @@ init_loopfilter(void)
 	 * file, so set the state to S_NSET.
 	 */
 	rstclock(S_NSET, current_time, 0);
+	sys_jitter = LOGTOD(sys_precision);
 }
 
 /*
@@ -193,12 +196,10 @@ init_loopfilter(void)
 int
 local_clock(
 	struct peer *peer,	/* synch source peer structure */
-	double fp_offset,	/* clock offset (s) */
-	double epsil		/* jittter (square s*s) */
+	double fp_offset	/* clock offset (s) */
 	)
 {
 	u_long mu;		/* interval since last update (s) */
-	double oerror;		/* previous error estimate */
 	double flladj;		/* FLL frequency adjustment (ppm) */
 	double plladj;		/* PLL frequency adjustment (ppm) */
 	double clock_frequency;	/* clock frequency adjustment (ppm) */
@@ -212,8 +213,8 @@ local_clock(
 #ifdef DEBUG
 	if (debug)
 		printf(
-		    "local_clock: assocID %d offset %.6f jitter %.6f state %d\n",
-		    peer->associd, fp_offset, SQRT(epsil), state);
+		    "local_clock: assocID %d offset %.9f state %d\n",
+		    peer->associd, fp_offset, state);
 #endif
 #ifdef LOCKCLOCK
 	sys_rootdispersion = peer->rootdispersion;
@@ -221,7 +222,7 @@ local_clock(
 
 #else /* LOCKCLOCK */
 	if (!ntp_enable) {
-		record_loop_stats(fp_offset, drift_comp, SQRT(epsil),
+		record_loop_stats(fp_offset, drift_comp, sys_error,
 		    clock_stability, sys_poll);
 		return (0);
 	}
@@ -266,35 +267,17 @@ local_clock(
 			    fp_offset);
 			printf("ntpd: time slew %+.6fs\n", fp_offset);
 		}
-		record_loop_stats(fp_offset, drift_comp, SQRT(epsil),
+		record_loop_stats(fp_offset, drift_comp, sys_error,
 		    clock_stability, sys_poll);
 		exit (0);
 	}
 
 	/*
-	 * If the clock has never been set, set it and initialize the
-	 * discipline parameters. We then switch to frequency mode to
-	 * speed the inital convergence process. If lucky, after an hour
-	 * the ntp.drift file is created and initialized and we don't
-	 * get here again.
-	 */
-	if (state == S_NSET) {
-		if (fabs(fp_offset) > clock_max && clock_max > 0) {
-			step_systime(fp_offset);
-			NLOG(NLOG_SYNCEVENT | NLOG_SYSEVENT)
-			    msyslog(LOG_NOTICE, "time reset %+.6f s",
-			    fp_offset);
-		}
-		rstclock(S_FREQ, peer->epoch, 0);
-		return (1);
-	}
-
-	/*
-	 * Update the jitter estimate.
-	 */
-	oerror = sys_jitter;
-	dtemp = SQUARE(sys_jitter);
-	sys_jitter = SQRT(dtemp + (epsil - dtemp) / CLOCK_AVG);
+         * Update the jitter estimate.
+         */
+	etemp = SQUARE(sys_jitter);
+	dtemp = SQUARE(fp_offset - last_offset);
+	sys_jitter = SQRT(etemp + (dtemp - etemp) / CLOCK_AVG);
 
 	/*
 	 * The huff-n'-puff filter finds the lowest delay in the recent
@@ -332,10 +315,6 @@ local_clock(
 	 * these actions interact with the command line options.
 	 */
 	retval = 0;
-	if (sys_poll > peer->maxpoll)
-		sys_poll = peer->maxpoll;
-	else if (sys_poll < peer->minpoll)
-		sys_poll = peer->minpoll;
 	clock_frequency = flladj = plladj = 0;
 	mu = peer->epoch - last_time;
 	if (fabs(fp_offset) > clock_max && clock_max > 0) {
@@ -349,7 +328,7 @@ local_clock(
 		 * to S_FREQ state.
 		 */
 		case S_TSET:
-			state = S_FREQ;
+			rstclock(S_FREQ, peer->epoch, fp_offset);
 			break;
 
 		/*
@@ -371,6 +350,7 @@ local_clock(
 		case S_FREQ:
 			if (mu < clock_minstep)
 				return (0);
+
 			/* fall through to S_SPIK */
 
 		/*
@@ -384,22 +364,33 @@ local_clock(
 			/* fall through to default */
 
 		/*
-		 * We get here directly in S_FSET state and indirectly
-		 * from S_FREQ and S_SPIK states. The clock is either
-		 * reset or shaken, but never stirred.
+		 * We get here directly in S_NSET and S_FSET states and
+		 * indirectly from S_FREQ and S_SPIK states. The clock
+		 * is either reset or shaken, but never stirred.
 		 */
 		default:
 			step_systime(fp_offset);
-			NLOG(NLOG_SYNCEVENT | NLOG_SYSEVENT)
-			    msyslog(LOG_NOTICE, "time reset %+.6f s",
+			msyslog(LOG_NOTICE, "time reset %+.6f s",
 			    fp_offset);
-			rstclock(S_TSET, peer->epoch, 0);
+			reinit_timer();
+			if (state == S_NSET)
+				rstclock(S_FREQ, peer->epoch, 0);
+			else
+				rstclock(S_TSET, peer->epoch, 0);
 			retval = 1;
 			break;
 		}
 	} else {
 		switch (state) {
 
+		/*
+		 * If the frequency has not been initialized from the
+		 * file, drop everything until it is.
+		 */
+		case S_NSET:
+			rstclock(S_FREQ, peer->epoch, fp_offset);
+			break;
+
 		/*
 		 * In S_FSET state this is the first update. Adjust the
 		 * phase, but don't adjust the frequency until the next
@@ -443,37 +434,42 @@ local_clock(
 		default:
 			allow_panic = FALSE;
 			dtemp = fabs(fp_offset - last_offset);
-			if (dtemp > CLOCK_SGATE * oerror && mu <
-			    ULOGTOD(sys_poll + 1)) {
+
+			if (dtemp > CLOCK_SGATE * sys_error && mu <
+			    (u_long) ULOGTOD(sys_poll + 1)) {
 #ifdef DEBUG
 				if (debug)
 					printf(
 				    "local_clock: popcorn %.6f %.6f\n",
-					    dtemp, oerror);
+					    dtemp, sys_error);
 #endif
 				last_offset = fp_offset;
 				return (0);
 			}
 
+
 			/*
-			 * Compute the FLL and PLL frequency gains,
-			 * which depend on the poll interval, update
-			 * interval and Allan intercept. For the FLL,
-			 * the averaging interval is clamped not less
-			 * than the Allan intercept and the weight
-			 * proportional to the poll interval from zero
-			 * to unity at the Allan intercept. For the PLL,
-			 * the averaging interval is clamped not greater
-			 * than the poll interval. Particularly for the
-			 * PLL, these measures allow oversampling, but
-			 * not undersampling and insure stability even
-			 * when the rules of fair engagement are broken.
+			 * The FLL and PLL frequency gain constants
+			 * depend on the poll interval and Allan
+			 * intercept. The PLL constant is calculated
+			 * throughout the poll interval range, but the
+			 * update interval is clamped so as not to
+			 * exceed the poll interval. The FLL gain is
+			 * zero below one-half the Allan intercept and
+			 * unity at MAXPOLL. It decreases as 1 /
+			 * (MAXPOLL + 1 - poll interval) in a feeble
+			 * effort to match the loop stiffness to the
+			 * Allan wobble. Particularly for the PLL, these
+			 * measures allow oversampling, but not
+			 * undersampling and insure stability even when
+			 * the rules of fair engagement are broken.
 			 */
-			etemp = min(1, sqrt(mu) / allan_xpt);
-			dtemp = max(mu, allan_xpt);
-			flladj = (fp_offset - clock_offset) * etemp /
-			    (CLOCK_FLL * dtemp);
-			etemp = min(mu, ULOGTOD(sys_poll));
+			if (ULOGTOD(sys_poll) > allan_xpt / 2) {
+				dtemp = NTP_MAXPOLL + 1 - sys_poll;
+				flladj = (fp_offset - clock_offset) /
+				    (max(mu, allan_xpt) * dtemp);
+			}
+			etemp = min(mu, (u_long)ULOGTOD(sys_poll));
 			dtemp = 4 * CLOCK_PLL * ULOGTOD(sys_poll);
 			plladj = fp_offset * etemp / (dtemp * dtemp);
 			last_time = peer->epoch;
@@ -529,7 +525,7 @@ local_clock(
 				ntv.freq = (int32)((clock_frequency +
 				    drift_comp) * 65536e6);
 			}
-			ntv.esterror = (u_int32)(sys_jitter * 1e6);
+			ntv.esterror = (u_int32)(sys_error * 1e6);
 			ntv.maxerror = (u_int32)((sys_rootdelay / 2 +
 			    sys_rootdispersion) * 1e6);
 			ntv.status = STA_PLL;
@@ -586,6 +582,12 @@ local_clock(
 				    "kernel time sync disabled %04x",
 				    ntv.status);
 			ntv.status &= ~(STA_PPSFREQ | STA_PPSTIME);
+		} else {
+			if (ntv.status != pll_status)
+				NLOG(NLOG_SYNCEVENT | NLOG_SYSEVENT)
+				    msyslog(LOG_NOTICE,
+				    "kernel time sync enabled %04x",
+				    ntv.status);
 		}
 		pll_status = ntv.status;
 		if (pll_nano)
@@ -601,9 +603,9 @@ local_clock(
 		if (ntv.status & STA_PPSTIME) {
 			pps_control = current_time;
 			if (pll_nano)
-				sys_jitter = ntv.jitter / 1e9;
+				sys_error = ntv.jitter / 1e9;
 			else
-				sys_jitter = ntv.jitter / 1e6;
+				sys_error = ntv.jitter / 1e6;
 		}
 	}
 #endif /* KERNEL_PLL */
@@ -615,20 +617,22 @@ local_clock(
 	 * drift_comp is a sham and used only for updating the drift
 	 * file and for billboard eye candy.
 	 */
-	etemp = clock_frequency + flladj + plladj;
-	drift_comp += etemp;
-	if (drift_comp > NTP_MAXFREQ)
+	dtemp = clock_frequency + flladj + plladj;
+	etemp = drift_comp + dtemp;
+	if (etemp > NTP_MAXFREQ)
 		drift_comp = NTP_MAXFREQ;
-	else if (drift_comp <= -NTP_MAXFREQ)
+	else if (etemp <= -NTP_MAXFREQ)
 		drift_comp = -NTP_MAXFREQ;
+	else
+		drift_comp = etemp;
 	if (fabs(etemp) > NTP_MAXFREQ)
-		msyslog(LOG_NOTICE,
+		NLOG(NLOG_SYNCEVENT | NLOG_SYSEVENT)
+		    msyslog(LOG_NOTICE,
 		    "frequency error %.0f PPM exceeds tolerance %.0f PPM",
 		    etemp * 1e6, NTP_MAXFREQ * 1e6);
-
-	dtemp = SQUARE(clock_stability);
-	etemp = SQUARE(etemp) - dtemp;
-	clock_stability = SQRT(dtemp + etemp / CLOCK_AVG);
+	etemp = SQUARE(clock_stability);
+	dtemp = SQUARE(dtemp * ULOGTOD(sys_poll));
+	clock_stability = SQRT(etemp + (dtemp - etemp) / CLOCK_AVG);
 
 	/*
 	 * In SYNC state, adjust the poll interval. The trick here is to
@@ -640,14 +644,14 @@ local_clock(
 	 * helps calm the dance. Works best using burst mode.
 	 */
 	if (state == S_SYNC) {
-		if (sys_jitter > ULOGTOD(sys_poll) * clock_stability &&
-		    fabs(clock_offset) < CLOCK_PGATE * sys_jitter) {
+		if (sys_jitter > clock_stability && fabs(clock_offset) <
+		    CLOCK_PGATE * sys_jitter) {
 			tc_counter += sys_poll;
 			if (tc_counter > CLOCK_LIMIT) {
 				tc_counter = CLOCK_LIMIT;
 				if (sys_poll < peer->maxpoll) {
 					tc_counter = 0;
-					sys_poll++;
+ 					sys_poll++;
 				}
 			}
 		} else {
@@ -666,11 +670,11 @@ local_clock(
 	 * Update the system time variables.
 	 */
 	dtemp = peer->disp + (current_time - peer->epoch) * clock_phi +
-	    sys_jitter + fabs(last_offset);
+	    sys_error + fabs(last_offset);
 	if (!(peer->flags & FLAG_REFCLOCK) && dtemp < MINDISPERSE)
 		dtemp = MINDISPERSE;
 	sys_rootdispersion = peer->rootdispersion + dtemp;
-	record_loop_stats(last_offset, drift_comp, sys_jitter,
+	record_loop_stats(last_offset, drift_comp, sys_error,
 	    clock_stability, sys_poll);
 
 #ifdef DEBUG
@@ -697,7 +701,6 @@ adj_host_clock(
 	)
 {
 	double	adjustment;
-	double	dtemp;
 
 	/*
 	 * Update the dispersion since the last update. In contrast to
@@ -744,16 +747,10 @@ adj_host_clock(
 	}
 
 	/*
-	 * This ugly bit of business is necessary in order to move the
-	 * pole frequency higher as the frequency gain in FLL mode is
-	 * increased. This is necessary to keep the overshoot to less
-	 * than a few percent. Don't look too closely here; this is
-	 * somewhat of a black art.
+	 * Implement the phase and frequency adjustments. Note the
+	 * black art formerly practiced here has been whitewashed.
 	 */
-	dtemp = sys_poll;
-	if (sys_poll > NTP_MINDPOLL)
-		dtemp *= 1.2;
-	adjustment = clock_offset / (CLOCK_PLL * pow(2, dtemp));
+	adjustment = clock_offset / (CLOCK_PLL * ULOGTOD(sys_poll));
 	clock_offset -= adjustment;
 	adj_systime(adjustment + drift_comp);
 #endif /* LOCKCLOCK */
@@ -777,8 +774,8 @@ rstclock(
 	last_offset = clock_offset = offset;
 #ifdef DEBUG
 	if (debug)
-		printf("clock_filter: at %lu state %d\n", last_time,
-		    trans);
+		printf("local_clock: time %lu offset %.6f state %d\n",
+		    last_time, last_offset, trans);
 #endif
 }
 
@@ -926,7 +923,7 @@ loop_config(
 			(void)ntp_adjtime(&ntv);
 		}
 #endif /* KERNEL_PLL */
-#endif LOCKCLOCK
+#endif /* LOCKCLOCK */
 		break;
 
 	/*
@@ -962,6 +959,11 @@ loop_config(
 			sys_huffpuff[i] = 1e9;
 		sys_mindly = 1e9;
 		break;
+
+	case LOOP_FREQ:			/* initial frequency */	
+		drift_comp = freq / 1e6;
+		rstclock(S_FSET, current_time, 0);
+		break;
 	}
 }
 
