@@ -226,8 +226,9 @@ refclock_newpeer(
 	peer->refclktype = clktype;
 	peer->refclkunit = (u_char)unit;
 	peer->flags |= FLAG_REFCLOCK;
-	peer->maxpoll = peer->minpoll;
 	peer->stratum = STRATUM_REFCLOCK;
+	peer->hpoll = peer->minpoll;
+	peer->ppoll = peer->maxpoll;
 	pp->type = clktype;
 	pp->timestarted = current_time;
 
@@ -252,8 +253,6 @@ refclock_newpeer(
 		refclock_unpeer(peer);
 		return (0);
 	}
-	peer->hpoll = peer->minpoll;
-	peer->ppoll = peer->maxpoll;
 	peer->refid = pp->refid;
 	return (1);
 }
@@ -348,7 +347,7 @@ refclock_transmit(
 	peer->outdate = next;
 	if (peer->burst > 0)
 		peer->burst--;
-	poll_update(peer, 0);
+	poll_update(peer, peer->hpoll);
 }
 
 
@@ -470,13 +469,14 @@ refclock_sample(
 	 * Copy the raw offsets and sort into ascending order. Don't do
 	 * anything if the buffer is empty.
 	 */
-	if (pp->codeproc == pp->coderecv)
-		return (0);
 	n = 0;
 	while (pp->codeproc != pp->coderecv) {
-		off[n++] = pp->filter[pp->codeproc];
 		pp->codeproc = (pp->codeproc + 1) % MAXSTAGE;
+		off[n] = pp->filter[pp->codeproc];
+		n++;
 	}
+	if (n == 0)
+		return (0);
 	if (n > 1)
 		qsort((char *)off, (size_t)n, sizeof(double), refclock_cmpl_fp);
 
@@ -542,9 +542,8 @@ refclock_receive(
 	 * the median filter samples and give the data to the clock
 	 * filter.
 	 */
-	peer->received++;
 	pp = peer->procptr;
-	peer->processed++;
+	peer->received++;
 	peer->timereceived = current_time;
 	peer->leap = pp->leap;
 	if (peer->leap == LEAP_NOTINSYNC) {
@@ -1006,11 +1005,12 @@ refclock_control(
 		if (in->haveflags & CLK_HAVEVAL2)
 			pp->refid = in->fudgeval2;
 		peer->stratum = pp->stratum;
-		if (peer->stratum == 0)
+		if (peer->stratum == STRATUM_REFCLOCK || peer->stratum ==
+		    STRATUM_UNSPEC)
 			peer->refid = pp->refid;
 		else
-			/* Here it is IPv4 address, so we don't have problem with REFID case */
-			peer->refid = ((struct sockaddr_in*)&peer->srcadr)->sin_addr.s_addr;
+			peer->refid = ((struct
+			    sockaddr_in*)&peer->srcadr)->sin_addr.s_addr;
 		if (in->haveflags & CLK_HAVEFLAG1) {
 			pp->sloppyclockflag &= ~CLK_FLAG1;
 			pp->sloppyclockflag |= in->flags & CLK_FLAG1;
