@@ -8,6 +8,10 @@
 #include "ntp_select.h"
 #include "ntp_io.h"
 #include "ntp_stdlib.h"
+/* Don't include ISC's version of IPv6 variables and structures */
+#define ISC_IPV6_H 1
+#include "isc/net.h"
+#include "isc/result.h"
 
 #include <ctype.h>
 #include <signal.h>
@@ -20,10 +24,10 @@
 # define closesocket close
 #endif /* SYS_WINNT */
 
-#ifdef HAVE_LIBREADLINE
+#if defined(HAVE_LIBREADLINE) || defined (HAVE_LIBEDIT)
 # include <readline/readline.h>
 # include <readline/history.h>
-#endif /* HAVE_LIBREADLINE */
+#endif /* HAVE_LIBREADLINE || HAVE_LIBEDIT */
 
 #ifdef SYS_VXWORKS
 /* vxWorks needs mode flag -casey*/
@@ -164,6 +168,7 @@ static	char currenthost[LENHOSTNAME];			/* current host name */
 int showhostnames = 1;					/* show host names by default */
 
 static	int ai_fam_templ;				/* address family */
+static	int ai_fam_default;				/* default address family */
 static	SOCKET sockfd;					/* fd socket is opened on */
 static	int havehost = 0;				/* set to 1 when host open */
 int s_port = 0;
@@ -173,8 +178,6 @@ char password[9];
 #endif /* SYS_WINNT || SYS_VXWORKS */
 
 #ifdef SYS_WINNT
-WORD wVersionRequested;
-WSADATA wsaData;
 DWORD NumberOfBytesWritten;
 
 HANDLE	TimerThreadHandle = NULL;	/* 1998/06/03 - Used in ntplib/machines.c */
@@ -298,7 +301,21 @@ ntpdcmain(
 	taskPrioritySet(taskIdSelf(), 100 );
 #endif
 
+#ifdef SYS_WINNT
+	if (!Win32InitSockets())
+	{
+		fprintf(stderr, "No useable winsock.dll:");
+		exit(1);
+	}
+#endif /* SYS_WINNT */
+
+	/* Check to see if we have IPv6. Otherwise force the -4 flag */
+	if (isc_net_probeipv6() != ISC_R_SUCCESS) {
+		ai_fam_default = AF_INET;
+	}
+
 	progname = argv[0];
+	ai_fam_templ = ai_fam_default;
 	while ((c = ntp_getopt(argc, argv, "46c:dilnps")) != EOF)
 	    switch (c) {
 		case '4':
@@ -365,14 +382,6 @@ ntpdcmain(
 	}
 	pktdatasize = INITDATASIZE;
 
-#ifdef SYS_WINNT
-	wVersionRequested = MAKEWORD(1,1);
-	if (WSAStartup(wVersionRequested, &wsaData)) {
-		fprintf(stderr, "No useable winsock.dll");
-		exit(1);
-	}
-#endif /* SYS_WINNT */
-
 	if (numcmds == 0) {
 		(void) openhost(chosts[0]);
 		getcmds();
@@ -432,7 +441,7 @@ openhost(
 	 * will return an "IPv4-mapped IPv6 address" address if you
 	 * give it an IPv4 address to lookup.
 	 */
-	sprintf(service, "%u", NTP_PORT);
+	strcpy(service, "ntp");
 	memset((char *)&hints, 0, sizeof(struct addrinfo));
 	hints.ai_family = ai_fam_templ;
 	hints.ai_protocol = IPPROTO_UDP;
@@ -440,7 +449,7 @@ openhost(
 	hints.ai_flags = AI_NUMERICHOST;
 
 	a_info = getaddrinfo(hname, service, &hints, &ai);
-	if (a_info == EAI_NONAME) {
+	if (a_info == EAI_NONAME || a_info == EAI_NODATA) {
 		hints.ai_flags = AI_CANONNAME;
 #ifdef AI_ADDRCONFIG
 		hints.ai_flags |= AI_ADDRCONFIG;
@@ -493,6 +502,7 @@ openhost(
 	{
 		int optionValue = SO_SYNCHRONOUS_NONALERT;
 		int err;
+
 		err = setsockopt(INVALID_SOCKET, SOL_SOCKET, SO_OPENTYPE, (char *)&optionValue, sizeof(optionValue));
 		if (err != NO_ERROR) {
 			(void) fprintf(stderr, "cannot open nonoverlapped sockets\n");
@@ -755,7 +765,7 @@ getresponse(
 		goto again;
 	}
 	/*
-	 * If we've received this before, toss it
+	 * If we've received this before, +toss it
 	 */
 	seq = INFO_SEQ(rpkt.auth_seq);
 	if (haveseq[seq]) {
@@ -779,7 +789,7 @@ getresponse(
 	/*
 	 * So far, so good.  Copy this data into the output array.
 	 */
-	if ((datap + datasize) > (pktdata + pktdatasize)) {
+	if ((datap + datasize + (pad * items)) > (pktdata + pktdatasize)) {
 		int offset = datap - pktdata;
 		growpktdata();
 	        *rdata = pktdata; /* might have been realloced ! */
@@ -1033,7 +1043,7 @@ again:
 static void
 getcmds(void)
 {
-#ifdef HAVE_LIBREADLINE
+#if defined(HAVE_LIBREADLINE) || defined(HAVE_LIBEDIT)
 	char *line;
 
 	for (;;) {
@@ -1042,7 +1052,7 @@ getcmds(void)
 		docmd(line);
 		free(line);
 	}
-#else /* not HAVE_LIBREADLINE */
+#else /* not (HAVE_LIBREADLINE || HAVE_LIBEDIT) */
 	char line[MAXLINE];
 
 	for (;;) {
@@ -1059,7 +1069,7 @@ getcmds(void)
 
 		docmd(line);
 	}
-#endif /* not HAVE_LIBREADLINE */
+#endif /* not HAVE_LIBREADLINE || HAVE_LIBEDIT */
 }
 
 
@@ -1096,7 +1106,7 @@ docmd(
 	int rval;
 	struct xcmd *xcmd;
 
-	ai_fam_templ = 0;
+	ai_fam_templ = ai_fam_default;
 	/*
 	 * Tokenize the command line.  If nothing on it, return.
 	 */
@@ -1374,7 +1384,6 @@ getnetnum(
 	int af
 	)
 {
-	int err;
 	int sockaddr_len;
 	struct addrinfo hints, *ai = NULL;
 
@@ -1395,7 +1404,7 @@ getnetnum(
 				    NI_NUMERICHOST); 
 		}
 		return 1;
-	} else if ((err = getaddrinfo(hname, "ntp", &hints, &ai)) == 0) {
+	} else if (getaddrinfo(hname, "ntp", &hints, &ai) == 0) {
 		memmove((char *)num, ai->ai_addr, ai->ai_addrlen);
 		if (fullhost != 0)
 			(void) strcpy(fullhost, ai->ai_canonname);
@@ -1442,7 +1451,7 @@ help(
 	int n;
 	struct xcmd *xcp;
 	char *cmd;
-	char *cmdsort[100];
+	const char *cmdsort[100];
 	int length[100];
 	int maxlength;
 	int numperline;
@@ -1452,10 +1461,10 @@ help(
 		n = 0;
 		for (xcp = builtins; xcp->keyword != 0; xcp++) {
 			if (*(xcp->keyword) != '?')
-			    cmdsort[n++] = (char *)xcp->keyword;
+			    cmdsort[n++] = xcp->keyword;
 		}
 		for (xcp = opcmds; xcp->keyword != 0; xcp++)
-		    cmdsort[n++] = (char *)xcp->keyword;
+		    cmdsort[n++] = xcp->keyword;
 
 #ifdef QSORT_USES_VOID_P
 		qsort(cmdsort, (size_t)n, sizeof(char *), helpsort);
