@@ -7,6 +7,7 @@
 #endif
 
 #include <stdio.h>
+#include <stddef.h>
 
 #include "ntpdc.h"
 #include "ntp_control.h"
@@ -292,7 +293,7 @@ peerlist(
 
 again:
 	res = doquery(impl_ver, REQ_PEER_LIST, 0, 0, 0, (char *)NULL, &items,
-		      &itemsize, (char **)&plist, 0, 
+		      &itemsize, (void *)&plist, 0, 
 		      sizeof(struct info_peer_list));
 	
 	if (res == INFO_ERR_IMPL && impl_ver == IMPL_XNTPD) {
@@ -300,14 +301,14 @@ again:
 		goto again;
 	}
 
-	if (res != 0 && items == 0)
+	if (res != 0)
 	    return;
 
 	if (!checkitems(items, fp))
 	    return;
 
 	if (!checkitemsize(itemsize, sizeof(struct info_peer_list)) &&
-	    !checkitemsize(itemsize, sizeof(struct info_peer_list) - ALT_LONG))
+	    !checkitemsize(itemsize, v4sizeof(struct info_peer_list)))
 	    return;
 
 	while (items > 0) {
@@ -382,7 +383,7 @@ dopeers(
 
 again:
 	res = doquery(impl_ver, REQ_PEER_LIST_SUM, 0, 0, 0, (char *)NULL,
-		      &items, &itemsize, (char **)&plist, 0, 
+		      &items, &itemsize, (void *)&plist, 0, 
 		      sizeof(struct info_peer_summary));
 	
 	if (res == INFO_ERR_IMPL && impl_ver == IMPL_XNTPD) {
@@ -390,15 +391,14 @@ again:
 		goto again;
 	}
 
-	if (res != 0 && items == 0)
+	if (res != 0)
 	    return;
 
 	if (!checkitems(items, fp))
 	    return;
 
 	if (!checkitemsize(itemsize, sizeof(struct info_peer_summary)) &&
-	    !checkitemsize(itemsize, sizeof(struct info_peer_summary) -
-	    ALT_2ADDR_LONG))
+	    !checkitemsize(itemsize, v4sizeof(struct info_peer_summary)))
 		return;
 
 	(void) fprintf(fp,
@@ -644,7 +644,7 @@ again:
 	if (impl_ver == IMPL_XNTPD)
 		sendsize = sizeof(struct info_peer_list);
 	else
-		sendsize = sizeof(struct info_peer_list) - ALT_LONG;
+		sendsize = v4sizeof(struct info_peer_list);
 
 	for (qitems = 0, pl = plist; qitems < min(pcmd->nargs, 4); qitems++) {
 		if (pcmd->argval[qitems].netnum.ss_family == AF_INET) {
@@ -667,21 +667,21 @@ again:
 
 	res = doquery(impl_ver, REQ_PEER_INFO, 0, qitems,
 		      sendsize, (char *)plist, &items,
-		      &itemsize, (char **)&pp, 0, sizeof(struct info_peer));
+		      &itemsize, (void *)&pp, 0, sizeof(struct info_peer));
 	
 	if (res == INFO_ERR_IMPL && impl_ver == IMPL_XNTPD) {
 		impl_ver = IMPL_XNTPD_OLD;
 		goto again;
 	}
 
-	if (res != 0 && items == 0)
+	if (res != 0)
 	    return;
 
 	if (!checkitems(items, fp))
 	    return;
 
 	if (!checkitemsize(itemsize, sizeof(struct info_peer)) &&
-	    !checkitemsize(itemsize, sizeof(struct info_peer) - ALT_2ADDR_LONG))
+	    !checkitemsize(itemsize, v4sizeof(struct info_peer)))
 	    return;
 
 	while (items-- > 0) {
@@ -716,7 +716,7 @@ again:
 	if (impl_ver == IMPL_XNTPD)
 		sendsize = sizeof(struct info_peer_list);
 	else
-		sendsize = sizeof(struct info_peer_list) - ALT_LONG;
+		sendsize = v4sizeof(struct info_peer_list);
 
 	memset((char *)plist, 0, sizeof(struct info_peer_list) * min(MAXARGS, 4));
 	for (qitems = 0, pl = plist; qitems < min(pcmd->nargs, 4); qitems++) {
@@ -740,7 +740,7 @@ again:
 
 	res = doquery(impl_ver, REQ_PEER_STATS, 0, qitems,
 		      sendsize, (char *)plist, &items,
-		      &itemsize, (char **)&pp, 0, 
+		      &itemsize, (void *)&pp, 0, 
 		      sizeof(struct info_peer_stats));
 	
 	if (res == INFO_ERR_IMPL && impl_ver == IMPL_XNTPD) {
@@ -748,15 +748,14 @@ again:
 		goto again;
 	}
 
-	if (res != 0 && items == 0)
+	if (res != 0)
 	    return;
 
 	if (!checkitems(items, fp))
 	    return;
 
 	if (!checkitemsize(itemsize, sizeof(struct info_peer_stats)) &&
-	    !checkitemsize(itemsize, sizeof(struct info_peer_stats) -
-	    ALT_2ADDR_INT))
+	    !checkitemsize(itemsize, v4sizeof(struct info_peer_stats)))
 	    return;
 
 	while (items-- > 0) {
@@ -839,7 +838,7 @@ loopinfo(
 
 again:
 	res = doquery(impl_ver, REQ_LOOP_INFO, 0, 0, 0, (char *)NULL,
-		      &items, &itemsize, (char **)&il, 0, 
+		      &items, &itemsize, (void *)&il, 0, 
 		      sizeof(struct info_loop));
 	
 	if (res == INFO_ERR_IMPL && impl_ver == IMPL_XNTPD) {
@@ -847,7 +846,7 @@ again:
 		goto again;
 	}
 
-	if (res != 0 && items == 0)
+	if (res != 0)
 	    return;
 
 	if (!check1item(items, fp))
@@ -902,7 +901,7 @@ sysinfo(
 
 again:
 	res = doquery(impl_ver, REQ_SYS_INFO, 0, 0, 0, (char *)NULL,
-		      &items, &itemsize, (char **)&is, 0,
+		      &items, &itemsize, (void *)&is, 0,
 		      sizeof(struct info_sys));
 	
 	if (res == INFO_ERR_IMPL && impl_ver == IMPL_XNTPD) {
@@ -910,14 +909,14 @@ again:
 		goto again;
 	}
 
-	if (res != 0 && items == 0)
+	if (res != 0)
 	    return;
 
 	if (!check1item(items, fp))
 	    return;
 
 	if (!checkitemsize(itemsize, sizeof(struct info_sys)) &&
-	    !checkitemsize(itemsize, sizeof(struct info_sys) - ALT_LONG))
+	    !checkitemsize(itemsize, v4sizeof(struct info_sys)))
 	    return;
 
 	memset((char *)&peeraddr, 0, sizeof(peeraddr));
@@ -999,7 +998,7 @@ sysstats(
 
 again:
 	res = doquery(impl_ver, REQ_SYS_STATS, 0, 0, 0, (char *)NULL,
-		      &items, &itemsize, (char **)&ss, 0, 
+		      &items, &itemsize, (void *)&ss, 0, 
 		      sizeof(struct info_sys_stats));
 	
 	if (res == INFO_ERR_IMPL && impl_ver == IMPL_XNTPD) {
@@ -1007,7 +1006,7 @@ again:
 		goto again;
 	}
 
-	if (res != 0 && items == 0)
+	if (res != 0)
 	    return;
 
 	if (!check1item(items, fp))
@@ -1065,7 +1064,7 @@ iostats(
 
 again:
 	res = doquery(impl_ver, REQ_IO_STATS, 0, 0, 0, (char *)NULL,
-		      &items, &itemsize, (char **)&io, 0, 
+		      &items, &itemsize, (void *)&io, 0, 
 		      sizeof(struct info_io_stats));
 	
 	if (res == INFO_ERR_IMPL && impl_ver == IMPL_XNTPD) {
@@ -1073,7 +1072,7 @@ again:
 		goto again;
 	}
 
-	if (res != 0 && items == 0)
+	if (res != 0)
 	    return;
 
 	if (!check1item(items, fp))
@@ -1127,7 +1126,7 @@ memstats(
 
 again:
 	res = doquery(impl_ver, REQ_MEM_STATS, 0, 0, 0, (char *)NULL,
-		      &items, &itemsize, (char **)&mem, 0, 
+		      &items, &itemsize, (void *)&mem, 0, 
 		      sizeof(struct info_mem_stats));
 	
 	if (res == INFO_ERR_IMPL && impl_ver == IMPL_XNTPD) {
@@ -1135,7 +1134,7 @@ again:
 		goto again;
 	}
 
-	if (res != 0 && items == 0)
+	if (res != 0)
 	    return;
 
 	if (!check1item(items, fp))
@@ -1158,9 +1157,9 @@ again:
 		       (u_long)ntohl(mem->demobilizations));
 
 	(void) fprintf(fp, "hash table counts:   ");
-	for (i = 0; i < HASH_SIZE; i++) {
+	for (i = 0; i < NTP_HASH_SIZE; i++) {
 		(void) fprintf(fp, "%4d", (int)mem->hashcount[i]);
-		if ((i % 8) == 7 && i != (HASH_SIZE-1)) {
+		if ((i % 8) == 7 && i != (NTP_HASH_SIZE-1)) {
 			(void) fprintf(fp, "\n                     ");
 		}
 	}
@@ -1186,7 +1185,7 @@ timerstats(
 
 again:
 	res = doquery(impl_ver, REQ_TIMER_STATS, 0, 0, 0, (char *)NULL,
-		      &items, &itemsize, (char **)&tim, 0, 
+		      &items, &itemsize, (void *)&tim, 0, 
 		      sizeof(struct info_timer_stats));
 	
 	if (res == INFO_ERR_IMPL && impl_ver == IMPL_XNTPD) {
@@ -1194,7 +1193,7 @@ again:
 		goto again;
 	}
 
-	if (res != 0 && items == 0)
+	if (res != 0)
 	    return;
 
 	if (!check1item(items, fp))
@@ -1298,7 +1297,7 @@ again:
 	if (impl_ver == IMPL_XNTPD)
 		sendsize = sizeof(struct conf_peer);
 	else
-		sendsize = sizeof(struct conf_peer) - ALT_LONG;
+		sendsize = v4sizeof(struct conf_peer);
 
 	items = pcmd->nargs;
 
@@ -1426,7 +1425,7 @@ again:
 	if (impl_ver == IMPL_XNTPD)
 		sendsize = sizeof(struct conf_unpeer);
 	else
-		sendsize = sizeof(struct conf_unpeer) - ALT_INT;
+		sendsize = v4sizeof(struct conf_unpeer);
 
 	for (qitems = 0, pl = plist; qitems < min(pcmd->nargs, 8); qitems++) {
 		if (pcmd->argval[0].netnum.ss_family == AF_INET) {
@@ -1607,7 +1606,7 @@ reslist(
 
 again:
 	res = doquery(impl_ver, REQ_GET_RESTRICT, 0, 0, 0, (char *)NULL,
-		      &items, &itemsize, (char **)&rl, 0, 
+		      &items, &itemsize, (void *)&rl, 0, 
 		      sizeof(struct info_restrict));
 	
 	if (res == INFO_ERR_IMPL && impl_ver == IMPL_XNTPD) {
@@ -1615,15 +1614,14 @@ again:
 		goto again;
 	}
 
-	if (res != 0 && items == 0)
+	if (res != 0)
 	    return;
 
 	if (!checkitems(items, fp))
 	    return;
 
 	if (!checkitemsize(itemsize, sizeof(struct info_restrict)) &&
-	    !checkitemsize(itemsize, sizeof(struct info_restrict) -
-	    ALT_2ADDR_LONG))
+	    !checkitemsize(itemsize, v4sizeof(struct info_restrict)))
 	    return;
 
 	(void) fprintf(fp,
@@ -1774,7 +1772,7 @@ again:
 	if (impl_ver == IMPL_XNTPD)
 		sendsize = sizeof(struct conf_restrict);
 	else
-		sendsize = sizeof(struct conf_restrict) - ALT_2ADDR_INT;
+		sendsize = v4sizeof(struct conf_restrict);
 
 	if (pcmd->argval[0].netnum.ss_family == AF_INET) {
 		cres.addr = GET_INADDR(pcmd->argval[0].netnum);
@@ -1901,18 +1899,18 @@ again:
 			  &items, &itemsize, &struct_star, 0, 
 			  sizeof(struct info_monitor));
 	
-	if (res != 0 && items == 0)
+	if (res != 0)
 	    return;
 
 	if (!checkitems(items, fp))
 	    return;
 
 	if (itemsize == sizeof(struct info_monitor_1) ||
-	    itemsize == sizeof(struct info_monitor_1) - ALT_2ADDR_LONG) {
+	    itemsize == v4sizeof(struct info_monitor_1)) {
 		struct info_monitor_1 *ml = (struct info_monitor_1 *) struct_star;
 
 		(void) fprintf(fp,
-			       "remote address          port local address      count m ver code avglen   first\n");
+			       "remote address          port local address      count m ver code avgint  lstint\n");
 		(void) fprintf(fp,
 			       "===============================================================================\n");
 		while (items > 0) {
@@ -1951,11 +1949,11 @@ again:
 			items--;
 		}
 	} else if (itemsize == sizeof(struct info_monitor) ||
-	    itemsize == sizeof(struct info_monitor) - ALT_LONG) {
+	    itemsize == v4sizeof(struct info_monitor)) {
 		struct info_monitor *ml = (struct info_monitor *) struct_star;
 
 		(void) fprintf(fp,
-			       "     address               port     count mode ver code avglen   first\n");
+			       "     address               port     count mode ver code avgint  lstint\n");
 		(void) fprintf(fp,
 			       "===============================================================================\n");
 		while (items > 0) {
@@ -2115,7 +2113,7 @@ again:
 	if (impl_ver == IMPL_XNTPD)
 		sendsize = sizeof(struct conf_unpeer);
 	else
-		sendsize = sizeof(struct conf_unpeer) - ALT_INT;
+		sendsize = v4sizeof(struct conf_unpeer);
 
 	for (qitems = 0, pl = plist; qitems < min(pcmd->nargs, 8); qitems++) {
 		if (pcmd->argval[qitems].netnum.ss_family == AF_INET) {
@@ -2262,7 +2260,7 @@ authinfo(
 
 again:
 	res = doquery(impl_ver, REQ_AUTHINFO, 0, 0, 0, (char *)NULL,
-		      &items, &itemsize, (char **)&ia, 0, 
+		      &items, &itemsize, (void *)&ia, 0, 
 		      sizeof(struct info_auth));
 	
 	if (res == INFO_ERR_IMPL && impl_ver == IMPL_XNTPD) {
@@ -2270,7 +2268,7 @@ again:
 		goto again;
 	}
 
-	if (res != 0 && items == 0)
+	if (res != 0)
 	    return;
 
 	if (!check1item(items, fp))
@@ -2320,7 +2318,7 @@ traps(
 
 again:
 	res = doquery(impl_ver, REQ_TRAPS, 0, 0, 0, (char *)NULL,
-		      &items, &itemsize, (char **)&it, 0, 
+		      &items, &itemsize, (void *)&it, 0, 
 		      sizeof(struct info_trap));
 	
 	if (res == INFO_ERR_IMPL && impl_ver == IMPL_XNTPD) {
@@ -2328,14 +2326,14 @@ again:
 		goto again;
 	}
 
-	if (res != 0 && items == 0)
+	if (res != 0)
 	    return;
 
 	if (!checkitems(items, fp))
 	    return;
 
 	if (!checkitemsize(itemsize, sizeof(struct info_trap)) &&
-	    !checkitemsize(itemsize, sizeof(struct info_trap) - ALT_2ADDR_INT))
+	    !checkitemsize(itemsize, v4sizeof(struct info_trap)))
 	    return;
 
 	for (i = 0; i < items; i++ ) {
@@ -2429,7 +2427,7 @@ again:
 	if (impl_ver == IMPL_XNTPD)
 		sendsize = sizeof(struct conf_trap);
 	else
-		sendsize = sizeof(struct conf_trap) - ALT_2ADDR_INT;
+		sendsize = v4sizeof(struct conf_trap);
 
 	if (pcmd->argval[0].netnum.ss_family == AF_INET) {
 		ctrap.trap_address = GET_INADDR(pcmd->argval[0].netnum);
@@ -2561,7 +2559,7 @@ ctlstats(
 
 again:
 	res = doquery(impl_ver, REQ_GET_CTLSTATS, 0, 0, 0, (char *)NULL,
-		      &items, &itemsize, (char **)&ic, 0, 
+		      &items, &itemsize, (void *)&ic, 0, 
 		      sizeof(struct info_control));
 	
 	if (res == INFO_ERR_IMPL && impl_ver == IMPL_XNTPD) {
@@ -2569,7 +2567,7 @@ again:
 		goto again;
 	}
 
-	if (res != 0 && items == 0)
+	if (res != 0)
 	    return;
 
 	if (!check1item(items, fp))
@@ -2637,14 +2635,14 @@ clockstat(
 again:
 	res = doquery(impl_ver, REQ_GET_CLOCKINFO, 0, qitems,
 		      sizeof(u_int32), (char *)clist, &items,
-		      &itemsize, (char **)&cl, 0, sizeof(struct info_clock));
+		      &itemsize, (void *)&cl, 0, sizeof(struct info_clock));
 	
 	if (res == INFO_ERR_IMPL && impl_ver == IMPL_XNTPD) {
 		impl_ver = IMPL_XNTPD_OLD;
 		goto again;
 	}
 
-	if (res != 0 && items == 0)
+	if (res != 0)
 	    return;
 
 	if (!checkitems(items, fp))
@@ -2810,14 +2808,14 @@ clkbug(
 again:
 	res = doquery(impl_ver, REQ_GET_CLKBUGINFO, 0, qitems,
 		      sizeof(u_int32), (char *)clist, &items,
-		      &itemsize, (char **)&cl, 0, sizeof(struct info_clkbug));
+		      &itemsize, (void *)&cl, 0, sizeof(struct info_clkbug));
 	
 	if (res == INFO_ERR_IMPL && impl_ver == IMPL_XNTPD) {
 		impl_ver = IMPL_XNTPD_OLD;
 		goto again;
 	}
 
-	if (res != 0 && items == 0)
+	if (res != 0)
 	    return;
 
 	if (!checkitems(items, fp))
@@ -2897,7 +2895,7 @@ kerninfo(
 
 again:
 	res = doquery(impl_ver, REQ_GET_KERNEL, 0, 0, 0, (char *)NULL,
-		      &items, &itemsize, (char **)&ik, 0, 
+		      &items, &itemsize, (void *)&ik, 0, 
 		      sizeof(struct info_kernel));
 
 	if (res == INFO_ERR_IMPL && impl_ver == IMPL_XNTPD) {
@@ -2905,7 +2903,7 @@ again:
 		goto again;
 	}
 
-	if (res != 0 && items == 0)
+	if (res != 0)
 	    return;
 	if (!check1item(items, fp))
 	    return;
