@@ -10,6 +10,10 @@
 #include "ntp_io.h"
 #include "ntp_select.h"
 #include "ntp_stdlib.h"
+/* Don't include ISC's version of IPv6 variables and structures */
+#define ISC_IPV6_H 1
+#include "isc/net.h"
+#include "isc/result.h"
 
 #include <ctype.h>
 #include <signal.h>
@@ -23,10 +27,10 @@
 #define closesocket close
 #endif /* SYS_WINNT */
 
-#ifdef HAVE_LIBREADLINE
+#if defined(HAVE_LIBREADLINE) || defined(HAVE_LIBEDIT)
 # include <readline/readline.h>
 # include <readline/history.h>
-#endif /* HAVE_LIBREADLINE */
+#endif /* HAVE_LIBREADLINE || HAVE_LIBEDIT */
 
 #ifdef SYS_VXWORKS
 /* vxWorks needs mode flag -casey*/
@@ -45,7 +49,7 @@ const char *prompt = "ntpq> ";	/* prompt to ask him about */
 /*
  * Keyid used for authenticated requests.  Obtained on the fly.
  */
-u_long info_auth_keyid = NTP_MAXKEY;
+u_long info_auth_keyid = 0;
 
 /*
  * Type of key md5
@@ -366,14 +370,13 @@ struct sockaddr_in hostaddr = { 0 };		/* host address */
 int showhostnames = 1;				/* show host names by default */
 
 int ai_fam_templ;				/* address family */
+int ai_fam_default;				/* default address family */
 SOCKET sockfd;					/* fd socket is opened on */
 int havehost = 0;				/* set to 1 when host open */
 int s_port = 0;
 struct servent *server_entry = NULL;		/* server entry for ntp */
 
 #ifdef SYS_WINNT
-WORD wVersionRequested;
-WSADATA wsaData;
 DWORD NumberOfBytesWritten;
 
 HANDLE	TimerThreadHandle = NULL;	/* 1998/06/03 - Used in ntplib/machines.c */
@@ -498,7 +501,21 @@ ntpqmain(
 	delay_time.l_ui = 0;
 	delay_time.l_uf = DEFDELAY;
 
+#ifdef SYS_WINNT
+	if (!Win32InitSockets())
+	{
+		fprintf(stderr, "No useable winsock.dll:");
+		exit(1);
+	}
+#endif /* SYS_WINNT */
+
+	/* Check to see if we have IPv6. Otherwise force the -4 flag */
+	if (isc_net_probeipv6() != ISC_R_SUCCESS) {
+		ai_fam_default = AF_INET;
+	}
+
 	progname = argv[0];
+	ai_fam_templ = ai_fam_default;
 	while ((c = ntp_getopt(argc, argv, "46c:dinp")) != EOF)
 	    switch (c) {
 		case '4':
@@ -549,14 +566,6 @@ ntpqmain(
 	    (void) signal_no_reset(SIGINT, abortcmd);
 #endif /* SYS_WINNT */
 
-#ifdef SYS_WINNT
-	wVersionRequested = MAKEWORD(1,1);
-	if (WSAStartup(wVersionRequested, &wsaData)) {
-		fprintf(stderr, "No useable winsock.dll");
-		exit(1);
-	}
-#endif /* SYS_WINNT */
-
 	if (numcmds == 0) {
 		(void) openhost(chosts[0]);
 		getcmds();
@@ -613,7 +622,7 @@ openhost(
 	 * will return an "IPv4-mapped IPv6 address" address if you
 	 * give it an IPv4 address to lookup.
 	 */
-	sprintf(service, "%u", NTP_PORT);
+	strcpy(service, "ntp");
 	memset((char *)&hints, 0, sizeof(struct addrinfo));
 	hints.ai_family = ai_fam_templ;
 	hints.ai_protocol = IPPROTO_UDP;
@@ -621,7 +630,7 @@ openhost(
 	hints.ai_flags = AI_NUMERICHOST;
 
 	a_info = getaddrinfo(hname, service, &hints, &ai);
-	if (a_info == EAI_NONAME) {
+	if (a_info == EAI_NONAME || a_info == EAI_NODATA) {
 		hints.ai_flags = AI_CANONNAME;
 #ifdef AI_ADDRCONFIG
 		hints.ai_flags |= AI_ADDRCONFIG;
@@ -675,6 +684,7 @@ openhost(
 	{
 		int optionValue = SO_SYNCHRONOUS_NONALERT;
 		int err;
+
 		err = setsockopt(INVALID_SOCKET, SOL_SOCKET, SO_OPENTYPE, (char *)&optionValue, sizeof(optionValue));
 		if (err != NO_ERROR) {
 			(void) fprintf(stderr, "cannot open nonoverlapped sockets\n");
@@ -1167,12 +1177,13 @@ sendrequest(
 		 * Get the keyid and the password if we don't have one.
 		 */
 		if (info_auth_keyid == 0) {
-			maclen = getkeyid("Keyid: ");
-			if (maclen == 0) {
+			int u_keyid = getkeyid("Keyid: ");
+			if (u_keyid == 0 || u_keyid > NTP_MAXKEY) {
 				(void) fprintf(stderr,
 				   "Invalid key identifier\n");
 				return 1;
 			}
+			info_auth_keyid = u_keyid;
 		}
 		if (!authistrusted(info_auth_keyid)) {
 			pass = getpass("MD5 Password: ");
@@ -1182,7 +1193,6 @@ sendrequest(
 				return (1);
 			}
 		}
-		info_auth_keyid = maclen;
 		authusekey(info_auth_keyid, info_auth_keytype, (const u_char *)pass);
 		authtrust(info_auth_keyid, 1);
 
@@ -1259,6 +1269,8 @@ doquery(
 			done = 1;
 			goto again;
 		}
+		if (numhosts > 1)
+			(void) fprintf(stderr, "server=%s ", currenthost);
 		switch(res) {
 		    case CERR_BADFMT:
 			(void) fprintf(stderr,
@@ -1315,7 +1327,7 @@ doquery(
 static void
 getcmds(void)
 {
-#ifdef HAVE_LIBREADLINE
+#if defined(HAVE_LIBREADLINE) || defined(HAVE_LIBEDIT)
         char *line;
 
         for (;;) {
@@ -1324,7 +1336,7 @@ getcmds(void)
                 docmd(line);
                 free(line);
         }
-#else /* not HAVE_LIBREADLINE */
+#else /* not (HAVE_LIBREADLINE || HAVE_LIBEDIT) */
         char line[MAXLINE];
 
         for (;;) {
@@ -1341,7 +1353,7 @@ getcmds(void)
 
                 docmd(line);
         }
-#endif /* not HAVE_LIBREADLINE */
+#endif /* not (HAVE_LIBREADLINE || HAVE_LIBEDIT) */
 }
 
 #ifndef SYS_WINNT /* Under NT cannot handle SIGINT, WIN32 spawns a handler */
@@ -1589,10 +1601,14 @@ getarg(
 				return 0;
 			}
 			if (isneg > numassoc) {
-				(void) fprintf(stderr,
-					       "***Association for `%s' unknown (max &%d)\n",
-					       str, numassoc);
-				return 0;
+				if (numassoc == 0) {
+					(void) fprintf(stderr,
+						       "***Association for `%s' unknown (max &%d)\n",
+						       str, numassoc);
+					return 0;
+				} else {
+					isneg = numassoc;
+				}
 			}
 			argp->uval = assoc_cache[isneg-1].assid;
 			break;
@@ -1653,7 +1669,6 @@ getnetnum(
 	int af
 	)
 {
-	int err;
 	int sockaddr_len;
 	struct addrinfo hints, *ai = NULL;
 
@@ -1672,12 +1687,11 @@ getnetnum(
 			getnameinfo((struct sockaddr *)num, sockaddr_len,
 					fullhost, sizeof(fullhost), NULL, 0,
 					NI_NUMERICHOST);
-
 		}
 		return 1;
-	} else if ((err = getaddrinfo(hname, "ntp", &hints, &ai)) == 0) {
+	} else if (getaddrinfo(hname, "ntp", &hints, &ai) == 0) {
 		memmove((char *)num, ai->ai_addr, ai->ai_addrlen);
-		if (fullhost != 0)
+		if (ai->ai_canonname != 0)
 		    (void) strcpy(fullhost, ai->ai_canonname);
 		return 1;
 	} else {
@@ -1902,8 +1916,8 @@ decodeint(
 {
 	if (*str == '0') {
 		if (*(str+1) == 'x' || *(str+1) == 'X')
-		    return hextoint(str+2, (u_long *)&val);
-		return octtoint(str, (u_long *)&val);
+		    return hextoint(str+2, (void *)&val);
+		return octtoint(str, (void *)&val);
 	}
 	return atoint(str, val);
 }
@@ -1982,7 +1996,7 @@ help(
 	int n;
 	struct xcmd *xcp;
 	char *cmd;
-	char *cmdsort[100];
+	const char *cmdsort[100];
 	int length[100];
 	int maxlength;
 	int numperline;
@@ -1992,10 +2006,10 @@ help(
 		n = 0;
 		for (xcp = builtins; xcp->keyword != 0; xcp++) {
 			if (*(xcp->keyword) != '?')
-			    cmdsort[n++] = (char *)xcp->keyword;
+			    cmdsort[n++] = xcp->keyword;
 		}
 		for (xcp = opcmds; xcp->keyword != 0; xcp++)
-		    cmdsort[n++] = (char *)xcp->keyword;
+		    cmdsort[n++] = xcp->keyword;
 
 #ifdef QSORT_USES_VOID_P
 		qsort(cmdsort, (size_t)n, sizeof(char *), helpsort);
@@ -2164,7 +2178,7 @@ host(
 	}
 
 	i = 0;
-	ai_fam_templ = 0;
+	ai_fam_templ = ai_fam_default;
 	if (pcmd->nargs == 2) {
 		if (!strcmp("-4", pcmd->argval[i].string))
 			ai_fam_templ = AF_INET;
@@ -2217,11 +2231,14 @@ keyid(
 	)
 {
 	if (pcmd->nargs == 0) {
-		if (info_auth_keyid > NTP_MAXKEY)
+		if (info_auth_keyid == 0)
 		    (void) fprintf(fp, "no keyid defined\n");
 		else
 		    (void) fprintf(fp, "keyid is %lu\n", (u_long)info_auth_keyid);
 	} else {
+		/* allow zero so that keyid can be cleared. */
+		if(pcmd->argval[0].uval > NTP_MAXKEY)
+		    (void) fprintf(fp, "Invalid key identifier\n");
 		info_auth_keyid = pcmd->argval[0].uval;
 	}
 }
@@ -2264,18 +2281,21 @@ passwd(
 {
 	char *pass;
 
-	if (info_auth_keyid > NTP_MAXKEY) {
-		info_auth_keyid = getkeyid("Keyid: ");
-		if (info_auth_keyid > NTP_MAXKEY) {
-			(void)fprintf(fp, "Keyid must be defined\n");
+	if (info_auth_keyid == 0) {
+		int u_keyid = getkeyid("Keyid: ");
+		if (u_keyid == 0 || u_keyid > NTP_MAXKEY) {
+			(void)fprintf(fp, "Invalid key identifier\n");
 			return;
 		}
+		info_auth_keyid = u_keyid;
 	}
 	pass = getpass("MD5 Password: ");
 	if (*pass == '\0')
 	    (void) fprintf(fp, "Password unchanged\n");
-	else
+	else {
 	    authusekey(info_auth_keyid, info_auth_keytype, (u_char *)pass);
+	    authtrust(info_auth_keyid, 1);
+	}
 }
 
 
